=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2010-11-11 03:53:12 +0000
+++ bzrlib/branch.py	2010-11-18 00:22:24 +0000
@@ -3355,7 +3355,7 @@
         if isinstance(format, remote.RemoteBranchFormat):
             format._ensure_real()
             return format._custom_format
-        return format                                                                                                  
+        return format
 
     @needs_write_lock
     def copy_content_into(self, revision_id=None):

=== modified file 'bzrlib/branchbuilder.py'
--- bzrlib/branchbuilder.py	2010-02-27 12:27:33 +0000
+++ bzrlib/branchbuilder.py	2010-11-18 08:13:01 +0000
@@ -21,6 +21,7 @@
     commit,
     errors,
     memorytree,
+    revision,
     )
 
 
@@ -186,7 +187,10 @@
         :return: The revision_id of the new commit
         """
         if parent_ids is not None:
-            base_id = parent_ids[0]
+            if len(parent_ids) == 0:
+                base_id = revision.NULL_REVISION
+            else:
+                base_id = parent_ids[0]
             if base_id != self._branch.last_revision():
                 self._move_branch_pointer(base_id,
                     allow_leftmost_as_ghost=allow_leftmost_as_ghost)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2010-11-04 17:48:47 +0000
+++ bzrlib/builtins.py	2010-11-18 00:22:24 +0000
@@ -4028,7 +4028,9 @@
         if ((remember or tree.branch.get_submit_branch() is None) and
              user_location is not None):
             tree.branch.set_submit_branch(other_branch.base)
-        _merge_tags_if_possible(other_branch, tree.branch)
+        # Merge tags (but don't set them in the master branch yet, the user
+        # might revert this merge).  Commit will propagate them.
+        _merge_tags_if_possible(other_branch, tree.branch, ignore_master=True)
         merger = _mod_merge.Merger.from_revision_ids(pb, tree,
             other_revision_id, base_revision_id, other_branch, base_branch)
         if other_path != '':

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2010-10-15 15:05:09 +0000
+++ bzrlib/bzrdir.py	2010-11-22 03:35:24 +0000
@@ -3254,36 +3254,6 @@
     tree_format='bzrlib.workingtree.WorkingTreeFormat5',
     )
 # The following un-numbered 'development' formats should always just be aliases.
-register_metadir(controldir.format_registry, 'development-rich-root',
-    'bzrlib.repofmt.groupcompress_repo.RepositoryFormatCHK1',
-    help='Current development format. Supports rich roots. Can convert data '
-        'to and from rich-root-pack (and anything compatible with '
-        'rich-root-pack) format repositories. Repositories and branches in '
-        'this format can only be read by bzr.dev. Please read '
-        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
-        'before use.',
-    branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
-    experimental=True,
-    alias=True,
-    hidden=True,
-    )
-register_metadir(controldir.format_registry, 'development5-subtree',
-    'bzrlib.repofmt.pack_repo.RepositoryFormatPackDevelopment2Subtree',
-    help='Development format, subtree variant. Can convert data to and '
-        'from pack-0.92-subtree (and anything compatible with '
-        'pack-0.92-subtree) format repositories. Repositories and branches in '
-        'this format can only be read by bzr.dev. Please read '
-        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
-        'before use.',
-    branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
-    experimental=True,
-    hidden=True,
-    alias=False,
-    )
-
-
 register_metadir(controldir.format_registry, 'development-subtree',
     'bzrlib.repofmt.groupcompress_repo.RepositoryFormat2aSubtree',
     help='Current development format, subtree variant. Can convert data to and '
@@ -3300,32 +3270,24 @@
                  # This current non-alias status is simply because we did not introduce a
                  # chk based subtree format.
     )
+register_metadir(controldir.format_registry, 'development5-subtree',
+    'bzrlib.repofmt.pack_repo.RepositoryFormatPackDevelopment2Subtree',
+    help='Development format, subtree variant. Can convert data to and '
+        'from pack-0.92-subtree (and anything compatible with '
+        'pack-0.92-subtree) format repositories. Repositories and branches in '
+        'this format can only be read by bzr.dev. Please read '
+        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
+        'before use.',
+    branch_format='bzrlib.branch.BzrBranchFormat7',
+    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
+    experimental=True,
+    hidden=True,
+    alias=False,
+    )
 
 # And the development formats above will have aliased one of the following:
-register_metadir(controldir.format_registry, 'development6-rich-root',
-    'bzrlib.repofmt.groupcompress_repo.RepositoryFormatCHK1',
-    help='pack-1.9 with 255-way hashed CHK inv, group compress, rich roots '
-        'Please read '
-        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
-        'before use.',
-    branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
-    hidden=True,
-    experimental=True,
-    )
-
-register_metadir(controldir.format_registry, 'development7-rich-root',
-    'bzrlib.repofmt.groupcompress_repo.RepositoryFormatCHK2',
-    help='pack-1.9 with 255-way hashed CHK inv, bencode revision, group compress, '
-        'rich roots. Please read '
-        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
-        'before use.',
-    branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
-    hidden=True,
-    experimental=True,
-    )
-
+
+# Finally, the current format.
 register_metadir(controldir.format_registry, '2a',
     'bzrlib.repofmt.groupcompress_repo.RepositoryFormat2a',
     help='First format for bzr 2.0 series.\n'

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2010-11-05 11:13:05 +0000
+++ bzrlib/config.py	2010-11-17 15:51:10 +0000
@@ -496,7 +496,8 @@
         config_id = self.config_id()
         for (section_name, section) in sections:
             for (name, value) in section.iteritems():
-                yield (name, value, section_name, config_id)
+                yield (name, parser._quote(value), section_name,
+                       config_id, parser)
 
     def _get_option_policy(self, section, option_name):
         """Return the policy for the given (section, option_name) pair."""
@@ -1042,7 +1043,8 @@
         config_id = self.config_id()
         for (section_name, section) in sections:
             for (name, value) in section.iteritems():
-                yield (name, value, section_name, config_id)
+                yield (name, value, section_name,
+                       config_id, branch_config._get_parser())
         # Then the global options
         for option in self._get_global_config()._get_options():
             yield option
@@ -1862,10 +1864,18 @@
         for c in self._get_configs(directory, scope):
             if displayed:
                 break
-            for (oname, value, section, conf_id) in c._get_options():
+            for (oname, value, section, conf_id, parser) in c._get_options():
                 if name == oname:
                     # Display only the first value and exit
-                    self.outf.write('%s\n' % (value))
+
+                    # FIXME: We need to use get_user_option to take policies
+                    # into account and we need to make sure the option exists
+                    # too (hence the two for loops), this needs a better API
+                    # -- vila 20101117
+                    value = c.get_user_option(name)
+                    # Quote the value appropriately
+                    value = parser._quote(value)
+                    self.outf.write('%s\n' % (value,))
                     displayed = True
                     break
         if not displayed:
@@ -1877,13 +1887,21 @@
         # avoid the delay introduced by the lazy regexp.
         name._compile_and_collapse()
         cur_conf_id = None
+        cur_section = None
         for c in self._get_configs(directory, scope):
-            for (oname, value, section, conf_id) in c._get_options():
+            for (oname, value, section, conf_id, parser) in c._get_options():
                 if name.search(oname):
                     if cur_conf_id != conf_id:
                         # Explain where the options are defined
                         self.outf.write('%s:\n' % (conf_id,))
                         cur_conf_id = conf_id
+                        cur_section = None
+                    if (section not in (None, 'DEFAULT')
+                        and cur_section != section):
+                        # Display the section if it's not the default (or only)
+                        # one.
+                        self.outf.write('  [%s]\n' % (section,))
+                        cur_section = section
                     self.outf.write('  %s = %s\n' % (oname, value))
 
     def _set_config_option(self, name, value, directory, scope):

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2010-08-25 13:02:32 +0000
+++ bzrlib/merge.py	2010-11-18 08:22:54 +0000
@@ -582,6 +582,7 @@
             elif len(lcas) == 1:
                 self.base_rev_id = list(lcas)[0]
             else: # len(lcas) > 1
+                self._is_criss_cross = True
                 if len(lcas) > 2:
                     # find_unique_lca can only handle 2 nodes, so we have to
                     # start back at the beginning. It is a shame to traverse
@@ -592,22 +593,30 @@
                 else:
                     self.base_rev_id = self.revision_graph.find_unique_lca(
                                             *lcas)
-                self._is_criss_cross = True
+                sorted_lca_keys = self.revision_graph.find_merge_order(                
+                    revisions[0], lcas)
+                if self.base_rev_id == _mod_revision.NULL_REVISION:
+                    self.base_rev_id = sorted_lca_keys[0]
+                
             if self.base_rev_id == _mod_revision.NULL_REVISION:
                 raise errors.UnrelatedBranches()
             if self._is_criss_cross:
                 trace.warning('Warning: criss-cross merge encountered.  See bzr'
                               ' help criss-cross.')
                 trace.mutter('Criss-cross lcas: %r' % lcas)
-                interesting_revision_ids = [self.base_rev_id]
-                interesting_revision_ids.extend(lcas)
+                if self.base_rev_id in lcas:
+                    trace.mutter('Unable to find unique lca. '
+                                 'Fallback %r as best option.' % self.base_rev_id)
+                interesting_revision_ids = set(lcas)
+                interesting_revision_ids.add(self.base_rev_id)
                 interesting_trees = dict((t.get_revision_id(), t)
                     for t in self.this_branch.repository.revision_trees(
                         interesting_revision_ids))
                 self._cached_trees.update(interesting_trees)
-                self.base_tree = interesting_trees.pop(self.base_rev_id)
-                sorted_lca_keys = self.revision_graph.find_merge_order(
-                    revisions[0], lcas)
+                if self.base_rev_id in lcas:
+                    self.base_tree = interesting_trees[self.base_rev_id]
+                else:
+                    self.base_tree = interesting_trees.pop(self.base_rev_id)
                 self._lca_trees = [interesting_trees[key]
                                    for key in sorted_lca_keys]
             else:

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2010-09-13 10:04:19 +0000
+++ bzrlib/msgeditor.py	2010-11-11 13:45:02 +0000
@@ -208,28 +208,25 @@
 
 def _create_temp_file_with_commit_template(infotext,
                                            ignoreline=DEFAULT_IGNORE_LINE,
-                                           start_message=None):
+                                           start_message=None,
+                                           tmpdir=None):
     """Create temp file and write commit template in it.
 
-    :param infotext:    Text to be displayed at bottom of message
-                        for the user's reference;
-                        currently similar to 'bzr status'.
-                        The text is already encoded.
+    :param infotext: Text to be displayed at bottom of message for the
+        user's reference; currently similar to 'bzr status'.  The text is
+        already encoded.
 
     :param ignoreline:  The separator to use above the infotext.
 
-    :param start_message:   The text to place above the separator, if any.
-                            This will not be removed from the message
-                            after the user has edited it.
-                            The string is already encoded
+    :param start_message: The text to place above the separator, if any.
+        This will not be removed from the message after the user has edited
+        it.  The string is already encoded
 
     :return:    2-tuple (temp file name, hasinfo)
     """
     import tempfile
     tmp_fileno, msgfilename = tempfile.mkstemp(prefix='bzr_log.',
-                                               dir='.',
-                                               text=True)
-    msgfilename = osutils.basename(msgfilename)
+                                               dir=tmpdir, text=True)
     msgfile = os.fdopen(tmp_fileno, 'w')
     try:
         if start_message is not None:

=== modified file 'bzrlib/plugins/launchpad/__init__.py'
--- bzrlib/plugins/launchpad/__init__.py	2010-10-11 14:10:29 +0000
+++ bzrlib/plugins/launchpad/__init__.py	2010-11-23 14:54:33 +0000
@@ -47,6 +47,8 @@
 lazy_import(globals(), """
 from bzrlib import (
     branch as _mod_branch,
+    errors,
+    ui,
     trace,
     )
 """)
@@ -367,6 +369,84 @@
 register_command(cmd_lp_propose_merge)
 
 
+class cmd_lp_find_proposal(Command):
+
+    __doc__ = """Find the proposal to merge this revision.
+
+    Finds the merge proposal(s) that discussed landing the specified revision.
+    This works only if the selected branch was the merge proposal target, and
+    if the merged_revno is recorded for the merge proposal.  The proposal(s)
+    are opened in a web browser.
+
+    Any revision involved in the merge may be specified-- the revision in
+    which the merge was performed, or one of the revisions that was merged.
+
+    So, to find the merge proposal that reviewed line 1 of README::
+
+      bzr lp-find-proposal -r annotate:README:1
+    """
+
+    takes_options = ['revision']
+
+    def run(self, revision=None):
+        from bzrlib.plugins.launchpad import lp_api
+        import webbrowser
+        b = _mod_branch.Branch.open_containing('.')[0]
+        pb = ui.ui_factory.nested_progress_bar()
+        b.lock_read()
+        try:
+            revno = self._find_merged_revno(revision, b, pb)
+            merged = self._find_proposals(revno, b, pb)
+            if len(merged) == 0:
+                raise BzrCommandError('No review found.')
+            trace.note('%d proposals(s) found.' % len(merged))
+            for mp in merged:
+                webbrowser.open(lp_api.canonical_url(mp))
+        finally:
+            b.unlock()
+            pb.finished()
+
+    def _find_merged_revno(self, revision, b, pb):
+        if revision is None:
+            return b.revno()
+        pb.update('Finding revision-id')
+        revision_id = revision[0].as_revision_id(b)
+        # a revno spec is necessarily on the mainline.
+        if self._is_revno_spec(revision[0]):
+            merging_revision = revision_id
+        else:
+            graph = b.repository.get_graph()
+            pb.update('Finding merge')
+            merging_revision = graph.find_lefthand_merger(
+                revision_id, b.last_revision())
+            if merging_revision is None:
+                raise errors.InvalidRevisionSpec(revision[0].user_spec, b)
+        pb.update('Finding revno')
+        return b.revision_id_to_revno(merging_revision)
+
+    def _find_proposals(self, revno, b, pb):
+        launchpad = lp_api.login(lp_registration.LaunchpadService())
+        pb.update('Finding Launchpad branch')
+        lpb = lp_api.LaunchpadBranch.from_bzr(launchpad, b,
+                                              create_missing=False)
+        pb.update('Finding proposals')
+        return list(lpb.lp.getMergeProposals(status=['Merged'],
+                                             merged_revnos=[revno]))
+
+
+    @staticmethod
+    def _is_revno_spec(spec):
+        try:
+            int(spec.user_spec)
+        except ValueError:
+            return False
+        else:
+            return True
+
+
+register_command(cmd_lp_find_proposal)
+
+
 def _register_directory():
     directories.register_lazy('lp:', 'bzrlib.plugins.launchpad.lp_directory',
                               'LaunchpadDirectory',

=== modified file 'bzrlib/plugins/launchpad/lp_api.py'
--- bzrlib/plugins/launchpad/lp_api.py	2010-03-05 08:55:12 +0000
+++ bzrlib/plugins/launchpad/lp_api.py	2010-11-23 14:54:33 +0000
@@ -23,6 +23,7 @@
 
 import os
 import re
+import urlparse
 
 from bzrlib import (
     branch,
@@ -47,6 +48,10 @@
     STAGING_SERVICE_ROOT,
     Launchpad,
     )
+try:
+    from launchpadlib.uris import LPNET_SERVICE_ROOT
+except ImportError:
+    LPNET_SERVICE_ROOT = 'https://api.launchpad.net/beta/'
 
 
 # Declare the minimum version of launchpadlib that we need in order to work.
@@ -75,7 +80,7 @@
 
 
 LAUNCHPAD_API_URLS = {
-    'production': 'https://api.launchpad.net/beta/',
+    'production': LPNET_SERVICE_ROOT,
     'edge': EDGE_SERVICE_ROOT,
     'staging': STAGING_SERVICE_ROOT,
     'dev': 'https://api.launchpad.dev/beta/',
@@ -101,6 +106,13 @@
         raise InvalidLaunchpadInstance(lp_instance)
 
 
+class NoLaunchpadBranch(errors.BzrError):
+    _fmt = 'No launchpad branch could be found for branch "%(url)s".'
+
+    def __init__(self, branch):
+        errors.BzrError.__init__(self, branch=branch, url=branch.base)
+
+
 def login(service, timeout=None, proxy_info=None):
     """Log in to the Launchpad API.
 
@@ -187,7 +199,7 @@
                            'bazaar.staging.launchpad.net')
 
     @classmethod
-    def from_bzr(cls, launchpad, bzr_branch):
+    def from_bzr(cls, launchpad, bzr_branch, create_missing=True):
         """Find a Launchpad branch from a bzr branch."""
         check_update = True
         for url in cls.candidate_urls(bzr_branch):
@@ -198,6 +210,8 @@
             if lp_branch is not None:
                 break
         else:
+            if not create_missing:
+                raise NoLaunchpadBranch(bzr_branch)
             lp_branch = cls.create_now(launchpad, bzr_branch)
             check_update = False
         return cls(lp_branch, bzr_branch.base, bzr_branch, check_update)
@@ -280,3 +294,13 @@
         if lp_branch:
             return lp_branch
     raise NotLaunchpadBranch(url)
+
+
+def canonical_url(object):
+    """Return the canonical URL for a branch."""
+    scheme, netloc, path, params, query, fragment = urlparse.urlparse(
+        str(object.self_link))
+    path = '/'.join(path.split('/')[2:])
+    netloc = netloc.replace('api.', 'code.')
+    return urlparse.urlunparse((scheme, netloc, path, params, query,
+                                fragment))

=== modified file 'bzrlib/plugins/launchpad/lp_propose.py'
--- bzrlib/plugins/launchpad/lp_propose.py	2010-08-25 10:20:41 +0000
+++ bzrlib/plugins/launchpad/lp_propose.py	2010-11-23 14:54:33 +0000
@@ -15,7 +15,6 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-import urlparse
 import webbrowser
 
 from bzrlib import (
@@ -27,6 +26,7 @@
     lp_api,
     lp_registration,
 )
+from bzrlib.plugins.launchpad.lp_api import canonical_url
 
 from lazr.restfulclient import errors as restful_errors
 
@@ -217,13 +217,3 @@
         old_tree):
         if c and k == 'file':
             yield str(path)
-
-
-def canonical_url(object):
-    """Return the canonical URL for a branch."""
-    scheme, netloc, path, params, query, fragment = urlparse.urlparse(
-        str(object.self_link))
-    path = '/'.join(path.split('/')[2:])
-    netloc = netloc.replace('api.', 'code.')
-    return urlparse.urlunparse((scheme, netloc, path, params, query,
-                                fragment))

=== modified file 'bzrlib/repofmt/groupcompress_repo.py'
--- bzrlib/repofmt/groupcompress_repo.py	2010-09-01 11:27:40 +0000
+++ bzrlib/repofmt/groupcompress_repo.py	2010-11-22 03:35:24 +0000
@@ -1339,19 +1339,15 @@
         yield record
 
 
-
-
-class RepositoryFormatCHK1(RepositoryFormatPack):
-    """A hashed CHK+group compress pack repository."""
+class RepositoryFormat2a(RepositoryFormatPack):
+    """A CHK repository that uses the bencode revision serializer."""
 
     repository_class = CHKInventoryRepository
     supports_external_lookups = True
     supports_chks = True
-    # For right now, setting this to True gives us InterModel1And2 rather
-    # than InterDifferingSerializer
     _commit_builder_class = PackRootCommitBuilder
     rich_root_data = True
-    _serializer = chk_serializer.chk_serializer_255_bigpage
+    _serializer = chk_serializer.chk_bencode_serializer
     _commit_inv_deltas = True
     # What index classes to use
     index_builder_class = BTreeBuilder
@@ -1368,52 +1364,6 @@
     pack_compresses = True
 
     def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir('development6-rich-root')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return ('Bazaar development format - group compression and chk inventory'
-                ' (needs bzr.dev from 1.14)\n')
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return ("Development repository format - rich roots, group compression"
-            " and chk inventories")
-
-
-class RepositoryFormatCHK2(RepositoryFormatCHK1):
-    """A CHK repository that uses the bencode revision serializer."""
-
-    _serializer = chk_serializer.chk_bencode_serializer
-
-    def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir('development7-rich-root')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return ('Bazaar development format - chk repository with bencode '
-                'revision serialization (needs bzr.dev from 1.16)\n')
-
-
-class RepositoryFormat2a(RepositoryFormatCHK2):
-    """A CHK repository that uses the bencode revision serializer.
-
-    This is the same as RepositoryFormatCHK2 but with a public name.
-    """
-
-    _serializer = chk_serializer.chk_bencode_serializer
-
-    def _get_matching_bzrdir(self):
         return bzrdir.format_registry.make_bzrdir('2a')
 
     def _ignore_setting_bzrdir(self, format):

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2010-08-25 10:20:41 +0000
+++ bzrlib/repofmt/knitrepo.py	2010-11-20 21:41:05 +0000
@@ -32,12 +32,11 @@
     xml7,
     )
 """)
-from bzrlib import (
-    symbol_versioning,
-    )
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.repository import (
     CommitBuilder,
+    InterRepository,
+    InterSameDataRepository,
     IsInWriteGroupError,
     MetaDirRepository,
     MetaDirRepositoryFormat,
@@ -510,3 +509,62 @@
     def get_format_description(self):
         """See RepositoryFormat.get_format_description()."""
         return "Knit repository format 4"
+
+
+class InterKnitRepo(InterSameDataRepository):
+    """Optimised code paths between Knit based repositories."""
+
+    @classmethod
+    def _get_repo_format_to_test(self):
+        return RepositoryFormatKnit1()
+
+    @staticmethod
+    def is_compatible(source, target):
+        """Be compatible with known Knit formats.
+
+        We don't test for the stores being of specific types because that
+        could lead to confusing results, and there is no need to be
+        overly general.
+        """
+        try:
+            are_knits = (isinstance(source._format, RepositoryFormatKnit) and
+                isinstance(target._format, RepositoryFormatKnit))
+        except AttributeError:
+            return False
+        return are_knits and InterRepository._same_model(source, target)
+
+    @needs_read_lock
+    def search_missing_revision_ids(self, revision_id=None, find_ghosts=True):
+        """See InterRepository.missing_revision_ids()."""
+        if revision_id is not None:
+            source_ids = self.source.get_ancestry(revision_id)
+            if source_ids[0] is not None:
+                raise AssertionError()
+            source_ids.pop(0)
+        else:
+            source_ids = self.source.all_revision_ids()
+        source_ids_set = set(source_ids)
+        # source_ids is the worst possible case we may need to pull.
+        # now we want to filter source_ids against what we actually
+        # have in target, but don't try to check for existence where we know
+        # we do not have a revision as that would be pointless.
+        target_ids = set(self.target.all_revision_ids())
+        possibly_present_revisions = target_ids.intersection(source_ids_set)
+        actually_present_revisions = set(
+            self.target._eliminate_revisions_not_present(possibly_present_revisions))
+        required_revisions = source_ids_set.difference(actually_present_revisions)
+        if revision_id is not None:
+            # we used get_ancestry to determine source_ids then we are assured all
+            # revisions referenced are present as they are installed in topological order.
+            # and the tip revision was validated by get_ancestry.
+            result_set = required_revisions
+        else:
+            # if we just grabbed the possibly available ids, then
+            # we only have an estimate of whats available and need to validate
+            # that against the revision records.
+            result_set = set(
+                self.source._eliminate_revisions_not_present(required_revisions))
+        return self.source.revision_ids_to_search_result(result_set)
+
+
+InterRepository.register_optimiser(InterKnitRepo)

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2010-09-28 18:51:47 +0000
+++ bzrlib/repofmt/pack_repo.py	2010-11-22 03:35:24 +0000
@@ -2894,12 +2894,10 @@
 class RepositoryFormatPackDevelopment2Subtree(RepositoryFormatPack):
     """A subtrees development repository.
 
-    This format should be retained until the second release after bzr 1.7.
+    This format should be retained in 2.3, to provide an upgrade path from this
+    to RepositoryFormat2aSubtree.  It can be removed in later releases.
 
     1.6.1-subtree[as it might have been] with B+Tree indices.
-
-    This is [now] retained until we have a CHK based subtree format in
-    development.
     """
 
     repository_class = KnitPackRepository

=== modified file 'bzrlib/repofmt/weaverepo.py'
--- bzrlib/repofmt/weaverepo.py	2010-08-25 10:20:41 +0000
+++ bzrlib/repofmt/weaverepo.py	2010-11-20 21:41:05 +0000
@@ -29,6 +29,7 @@
 from bzrlib import (
     xml5,
     graph as _mod_graph,
+    ui,
     )
 """)
 from bzrlib import (
@@ -38,7 +39,6 @@
     lockable_files,
     lockdir,
     osutils,
-    revision as _mod_revision,
     trace,
     urlutils,
     versionedfile,
@@ -48,6 +48,8 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.repository import (
     CommitBuilder,
+    InterRepository,
+    InterSameDataRepository,
     MetaDirVersionedFileRepository,
     MetaDirRepositoryFormat,
     Repository,
@@ -738,6 +740,116 @@
         paths = list(relpaths)
         return set([self._mapper.unmap(path) for path in paths])
 
+
+class InterWeaveRepo(InterSameDataRepository):
+    """Optimised code paths between Weave based repositories.
+
+    This should be in bzrlib/repofmt/weaverepo.py but we have not yet
+    implemented lazy inter-object optimisation.
+    """
+
+    @classmethod
+    def _get_repo_format_to_test(self):
+        return RepositoryFormat7()
+
+    @staticmethod
+    def is_compatible(source, target):
+        """Be compatible with known Weave formats.
+
+        We don't test for the stores being of specific types because that
+        could lead to confusing results, and there is no need to be
+        overly general.
+        """
+        try:
+            return (isinstance(source._format, (RepositoryFormat5,
+                                                RepositoryFormat6,
+                                                RepositoryFormat7)) and
+                    isinstance(target._format, (RepositoryFormat5,
+                                                RepositoryFormat6,
+                                                RepositoryFormat7)))
+        except AttributeError:
+            return False
+
+    @needs_write_lock
+    def copy_content(self, revision_id=None):
+        """See InterRepository.copy_content()."""
+        # weave specific optimised path:
+        try:
+            self.target.set_make_working_trees(self.source.make_working_trees())
+        except (errors.RepositoryUpgradeRequired, NotImplemented):
+            pass
+        # FIXME do not peek!
+        if self.source._transport.listable():
+            pb = ui.ui_factory.nested_progress_bar()
+            try:
+                self.target.texts.insert_record_stream(
+                    self.source.texts.get_record_stream(
+                        self.source.texts.keys(), 'topological', False))
+                pb.update('Copying inventory', 0, 1)
+                self.target.inventories.insert_record_stream(
+                    self.source.inventories.get_record_stream(
+                        self.source.inventories.keys(), 'topological', False))
+                self.target.signatures.insert_record_stream(
+                    self.source.signatures.get_record_stream(
+                        self.source.signatures.keys(),
+                        'unordered', True))
+                self.target.revisions.insert_record_stream(
+                    self.source.revisions.get_record_stream(
+                        self.source.revisions.keys(),
+                        'topological', True))
+            finally:
+                pb.finished()
+        else:
+            self.target.fetch(self.source, revision_id=revision_id)
+
+    @needs_read_lock
+    def search_missing_revision_ids(self, revision_id=None, find_ghosts=True):
+        """See InterRepository.missing_revision_ids()."""
+        # we want all revisions to satisfy revision_id in source.
+        # but we don't want to stat every file here and there.
+        # we want then, all revisions other needs to satisfy revision_id
+        # checked, but not those that we have locally.
+        # so the first thing is to get a subset of the revisions to
+        # satisfy revision_id in source, and then eliminate those that
+        # we do already have.
+        # this is slow on high latency connection to self, but as this
+        # disk format scales terribly for push anyway due to rewriting
+        # inventory.weave, this is considered acceptable.
+        # - RBC 20060209
+        if revision_id is not None:
+            source_ids = self.source.get_ancestry(revision_id)
+            if source_ids[0] is not None:
+                raise AssertionError()
+            source_ids.pop(0)
+        else:
+            source_ids = self.source._all_possible_ids()
+        source_ids_set = set(source_ids)
+        # source_ids is the worst possible case we may need to pull.
+        # now we want to filter source_ids against what we actually
+        # have in target, but don't try to check for existence where we know
+        # we do not have a revision as that would be pointless.
+        target_ids = set(self.target._all_possible_ids())
+        possibly_present_revisions = target_ids.intersection(source_ids_set)
+        actually_present_revisions = set(
+            self.target._eliminate_revisions_not_present(possibly_present_revisions))
+        required_revisions = source_ids_set.difference(actually_present_revisions)
+        if revision_id is not None:
+            # we used get_ancestry to determine source_ids then we are assured all
+            # revisions referenced are present as they are installed in topological order.
+            # and the tip revision was validated by get_ancestry.
+            result_set = required_revisions
+        else:
+            # if we just grabbed the possibly available ids, then
+            # we only have an estimate of whats available and need to validate
+            # that against the revision records.
+            result_set = set(
+                self.source._eliminate_revisions_not_present(required_revisions))
+        return self.source.revision_ids_to_search_result(result_set)
+
+
 _legacy_formats = [RepositoryFormat4(),
                    RepositoryFormat5(),
                    RepositoryFormat6()]
+
+
+InterRepository.register_optimiser(InterWeaveRepo)

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2010-10-19 19:10:35 +0000
+++ bzrlib/repository.py	2010-11-22 22:27:58 +0000
@@ -3352,35 +3352,20 @@
     'bzrlib.repofmt.pack_repo',
     'RepositoryFormatKnitPack6RichRoot',
     )
+format_registry.register_lazy(
+    'Bazaar repository format 2a (needs bzr 1.16 or later)\n',
+    'bzrlib.repofmt.groupcompress_repo',
+    'RepositoryFormat2a',
+    )
 
 # Development formats.
-# Obsolete but kept pending a CHK based subtree format.
+# Check their docstrings to see if/when they are obsolete.
 format_registry.register_lazy(
     ("Bazaar development format 2 with subtree support "
         "(needs bzr.dev from before 1.8)\n"),
     'bzrlib.repofmt.pack_repo',
     'RepositoryFormatPackDevelopment2Subtree',
     )
-
-# 1.14->1.16 go below here
-format_registry.register_lazy(
-    'Bazaar development format - group compression and chk inventory'
-        ' (needs bzr.dev from 1.14)\n',
-    'bzrlib.repofmt.groupcompress_repo',
-    'RepositoryFormatCHK1',
-    )
-
-format_registry.register_lazy(
-    'Bazaar development format - chk repository with bencode revision '
-        'serialization (needs bzr.dev from 1.16)\n',
-    'bzrlib.repofmt.groupcompress_repo',
-    'RepositoryFormatCHK2',
-    )
-format_registry.register_lazy(
-    'Bazaar repository format 2a (needs bzr 1.16 or later)\n',
-    'bzrlib.repofmt.groupcompress_repo',
-    'RepositoryFormat2a',
-    )
 format_registry.register_lazy(
     'Bazaar development format 8\n',
     'bzrlib.repofmt.groupcompress_repo',
@@ -3568,176 +3553,6 @@
         return InterRepository._same_model(source, target)
 
 
-class InterWeaveRepo(InterSameDataRepository):
-    """Optimised code paths between Weave based repositories.
-
-    This should be in bzrlib/repofmt/weaverepo.py but we have not yet
-    implemented lazy inter-object optimisation.
-    """
-
-    @classmethod
-    def _get_repo_format_to_test(self):
-        from bzrlib.repofmt import weaverepo
-        return weaverepo.RepositoryFormat7()
-
-    @staticmethod
-    def is_compatible(source, target):
-        """Be compatible with known Weave formats.
-
-        We don't test for the stores being of specific types because that
-        could lead to confusing results, and there is no need to be
-        overly general.
-        """
-        from bzrlib.repofmt.weaverepo import (
-                RepositoryFormat5,
-                RepositoryFormat6,
-                RepositoryFormat7,
-                )
-        try:
-            return (isinstance(source._format, (RepositoryFormat5,
-                                                RepositoryFormat6,
-                                                RepositoryFormat7)) and
-                    isinstance(target._format, (RepositoryFormat5,
-                                                RepositoryFormat6,
-                                                RepositoryFormat7)))
-        except AttributeError:
-            return False
-
-    @needs_write_lock
-    def copy_content(self, revision_id=None):
-        """See InterRepository.copy_content()."""
-        # weave specific optimised path:
-        try:
-            self.target.set_make_working_trees(self.source.make_working_trees())
-        except (errors.RepositoryUpgradeRequired, NotImplemented):
-            pass
-        # FIXME do not peek!
-        if self.source._transport.listable():
-            pb = ui.ui_factory.nested_progress_bar()
-            try:
-                self.target.texts.insert_record_stream(
-                    self.source.texts.get_record_stream(
-                        self.source.texts.keys(), 'topological', False))
-                pb.update('Copying inventory', 0, 1)
-                self.target.inventories.insert_record_stream(
-                    self.source.inventories.get_record_stream(
-                        self.source.inventories.keys(), 'topological', False))
-                self.target.signatures.insert_record_stream(
-                    self.source.signatures.get_record_stream(
-                        self.source.signatures.keys(),
-                        'unordered', True))
-                self.target.revisions.insert_record_stream(
-                    self.source.revisions.get_record_stream(
-                        self.source.revisions.keys(),
-                        'topological', True))
-            finally:
-                pb.finished()
-        else:
-            self.target.fetch(self.source, revision_id=revision_id)
-
-    @needs_read_lock
-    def search_missing_revision_ids(self, revision_id=None, find_ghosts=True):
-        """See InterRepository.missing_revision_ids()."""
-        # we want all revisions to satisfy revision_id in source.
-        # but we don't want to stat every file here and there.
-        # we want then, all revisions other needs to satisfy revision_id
-        # checked, but not those that we have locally.
-        # so the first thing is to get a subset of the revisions to
-        # satisfy revision_id in source, and then eliminate those that
-        # we do already have.
-        # this is slow on high latency connection to self, but as this
-        # disk format scales terribly for push anyway due to rewriting
-        # inventory.weave, this is considered acceptable.
-        # - RBC 20060209
-        if revision_id is not None:
-            source_ids = self.source.get_ancestry(revision_id)
-            if source_ids[0] is not None:
-                raise AssertionError()
-            source_ids.pop(0)
-        else:
-            source_ids = self.source._all_possible_ids()
-        source_ids_set = set(source_ids)
-        # source_ids is the worst possible case we may need to pull.
-        # now we want to filter source_ids against what we actually
-        # have in target, but don't try to check for existence where we know
-        # we do not have a revision as that would be pointless.
-        target_ids = set(self.target._all_possible_ids())
-        possibly_present_revisions = target_ids.intersection(source_ids_set)
-        actually_present_revisions = set(
-            self.target._eliminate_revisions_not_present(possibly_present_revisions))
-        required_revisions = source_ids_set.difference(actually_present_revisions)
-        if revision_id is not None:
-            # we used get_ancestry to determine source_ids then we are assured all
-            # revisions referenced are present as they are installed in topological order.
-            # and the tip revision was validated by get_ancestry.
-            result_set = required_revisions
-        else:
-            # if we just grabbed the possibly available ids, then
-            # we only have an estimate of whats available and need to validate
-            # that against the revision records.
-            result_set = set(
-                self.source._eliminate_revisions_not_present(required_revisions))
-        return self.source.revision_ids_to_search_result(result_set)
-
-
-class InterKnitRepo(InterSameDataRepository):
-    """Optimised code paths between Knit based repositories."""
-
-    @classmethod
-    def _get_repo_format_to_test(self):
-        from bzrlib.repofmt import knitrepo
-        return knitrepo.RepositoryFormatKnit1()
-
-    @staticmethod
-    def is_compatible(source, target):
-        """Be compatible with known Knit formats.
-
-        We don't test for the stores being of specific types because that
-        could lead to confusing results, and there is no need to be
-        overly general.
-        """
-        from bzrlib.repofmt.knitrepo import RepositoryFormatKnit
-        try:
-            are_knits = (isinstance(source._format, RepositoryFormatKnit) and
-                isinstance(target._format, RepositoryFormatKnit))
-        except AttributeError:
-            return False
-        return are_knits and InterRepository._same_model(source, target)
-
-    @needs_read_lock
-    def search_missing_revision_ids(self, revision_id=None, find_ghosts=True):
-        """See InterRepository.missing_revision_ids()."""
-        if revision_id is not None:
-            source_ids = self.source.get_ancestry(revision_id)
-            if source_ids[0] is not None:
-                raise AssertionError()
-            source_ids.pop(0)
-        else:
-            source_ids = self.source.all_revision_ids()
-        source_ids_set = set(source_ids)
-        # source_ids is the worst possible case we may need to pull.
-        # now we want to filter source_ids against what we actually
-        # have in target, but don't try to check for existence where we know
-        # we do not have a revision as that would be pointless.
-        target_ids = set(self.target.all_revision_ids())
-        possibly_present_revisions = target_ids.intersection(source_ids_set)
-        actually_present_revisions = set(
-            self.target._eliminate_revisions_not_present(possibly_present_revisions))
-        required_revisions = source_ids_set.difference(actually_present_revisions)
-        if revision_id is not None:
-            # we used get_ancestry to determine source_ids then we are assured all
-            # revisions referenced are present as they are installed in topological order.
-            # and the tip revision was validated by get_ancestry.
-            result_set = required_revisions
-        else:
-            # if we just grabbed the possibly available ids, then
-            # we only have an estimate of whats available and need to validate
-            # that against the revision records.
-            result_set = set(
-                self.source._eliminate_revisions_not_present(required_revisions))
-        return self.source.revision_ids_to_search_result(result_set)
-
-
 class InterDifferingSerializer(InterRepository):
 
     @classmethod
@@ -4079,8 +3894,6 @@
 
 InterRepository.register_optimiser(InterDifferingSerializer)
 InterRepository.register_optimiser(InterSameDataRepository)
-InterRepository.register_optimiser(InterWeaveRepo)
-InterRepository.register_optimiser(InterKnitRepo)
 
 
 class CopyConverter(object):

=== modified file 'bzrlib/smart/repository.py'
--- bzrlib/smart/repository.py	2010-05-14 13:36:34 +0000
+++ bzrlib/smart/repository.py	2010-11-16 06:06:11 +0000
@@ -30,7 +30,6 @@
     osutils,
     pack,
     ui,
-    versionedfile,
     )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.smart.request import (
@@ -39,7 +38,6 @@
     SuccessfulSmartServerResponse,
     )
 from bzrlib.repository import _strip_NULL_ghosts, network_format_registry
-from bzrlib.recordcounter import RecordCounter
 from bzrlib import revision as _mod_revision
 from bzrlib.versionedfile import (
     NetworkRecordStream,
@@ -506,8 +504,6 @@
         for record in substream:
             if record.storage_kind in ('chunked', 'fulltext'):
                 serialised = record_to_fulltext_bytes(record)
-            elif record.storage_kind == 'inventory-delta':
-                serialised = record_to_inventory_delta_bytes(record)
             elif record.storage_kind == 'absent':
                 raise ValueError("Absent factory for %s" % (record.key,))
             else:

=== modified file 'bzrlib/tag.py'
--- bzrlib/tag.py	2010-03-25 09:39:03 +0000
+++ bzrlib/tag.py	2010-11-18 00:30:51 +0000
@@ -28,7 +28,9 @@
 
 from bzrlib import (
     bencode,
+    cleanup,
     errors,
+    symbol_versioning,
     trace,
     )
 
@@ -57,7 +59,7 @@
     lookup_tag = _not_supported
     delete_tag = _not_supported
 
-    def merge_to(self, to_tags, overwrite=False):
+    def merge_to(self, to_tags, overwrite=False, ignore_master=False):
         # we never have anything to copy
         pass
 
@@ -177,7 +179,7 @@
             raise ValueError("failed to deserialize tag dictionary %r: %s"
                 % (tag_content, e))
 
-    def merge_to(self, to_tags, overwrite=False):
+    def merge_to(self, to_tags, overwrite=False, ignore_master=False):
         """Copy tags between repositories if necessary and possible.
 
         This method has common command-line behaviour about handling
@@ -188,11 +190,19 @@
 
         :param to_tags: Branch to receive these tags
         :param overwrite: Overwrite conflicting tags in the target branch
+        :param ignore_master: Do not modify the tags in the target's master
+            branch (if any).  Default is false (so the master will be updated).
+            New in bzr 2.3.
 
-        :returns: A list of tags that conflicted, each of which is
+        :returns: A set of tags that conflicted, each of which is
             (tagname, source_target, dest_target), or None if no copying was
             done.
         """
+        operation = cleanup.OperationWithCleanups(self._merge_to_operation)
+        return operation.run(to_tags, overwrite, ignore_master)
+
+    def _merge_to_operation(self, operation, to_tags, overwrite, ignore_master):
+        add_cleanup = operation.add_cleanup
         if self.branch == to_tags.branch:
             return
         if not self.branch.supports_tags():
@@ -203,15 +213,39 @@
             # no tags in the source, and we don't want to clobber anything
             # that's in the destination
             return
-        to_tags.branch.lock_write()
-        try:
-            dest_dict = to_tags.get_tag_dict()
-            result, conflicts = self._reconcile_tags(source_dict, dest_dict,
-                                                     overwrite)
-            if result != dest_dict:
-                to_tags._set_tag_dict(result)
-        finally:
-            to_tags.branch.unlock()
+        # We merge_to both master and child individually.
+        #
+        # It's possible for master and child to have differing sets of
+        # tags, in which case it's possible to have different sets of
+        # conflicts.  We report the union of both conflict sets.  In
+        # that case it's likely the child and master have accepted
+        # different tags from the source, which may be a surprising result, but
+        # the best we can do in the circumstances.
+        #
+        # Ideally we'd improve this API to report the different conflicts
+        # more clearly to the caller, but we don't want to break plugins
+        # such as bzr-builddeb that use this API.
+        add_cleanup(to_tags.branch.lock_write().unlock)
+        if ignore_master:
+            master = None
+        else:
+            master = to_tags.branch.get_master_branch()
+        if master is not None:
+            add_cleanup(master.lock_write().unlock)
+        conflicts = self._merge_to(to_tags, source_dict, overwrite)
+        if master is not None:
+            conflicts += self._merge_to(master.tags, source_dict,
+                overwrite)
+        # We use set() to remove any duplicate conflicts from the master
+        # branch.
+        return set(conflicts)
+
+    def _merge_to(self, to_tags, source_dict, overwrite):
+        dest_dict = to_tags.get_tag_dict()
+        result, conflicts = self._reconcile_tags(source_dict, dest_dict,
+                                                 overwrite)
+        if result != dest_dict:
+            to_tags._set_tag_dict(result)
         return conflicts
 
     def rename_revisions(self, rename_map):
@@ -249,6 +283,27 @@
         return result, conflicts
 
 
-def _merge_tags_if_possible(from_branch, to_branch):
-    from_branch.tags.merge_to(to_branch.tags)
+def _merge_tags_if_possible(from_branch, to_branch, ignore_master=False):
+    # Try hard to support merge_to implementations that don't expect
+    # 'ignore_master' (new in bzr 2.3).  First, if the flag isn't set then we
+    # can safely avoid passing ignore_master at all.
+    if not ignore_master:
+        from_branch.tags.merge_to(to_branch.tags)
+        return
+    # If the flag is set, try to pass it, but be ready to catch TypeError.
+    try:
+        from_branch.tags.merge_to(to_branch.tags, ignore_master=ignore_master)
+    except TypeError:
+        # Probably this implementation of 'merge_to' is from a plugin that
+        # doesn't expect the 'ignore_master' keyword argument (e.g. bzr-svn
+        # 1.0.4).  There's a small risk that the TypeError is actually caused
+        # by a completely different problem (which is why we don't catch it for
+        # the ignore_master=False case), but even then there's probably no harm
+        # in calling a second time.
+        symbol_versioning.warn(
+            symbol_versioning.deprecated_in((2,3)) % (
+                "Tags.merge_to (of %r) that doesn't accept ignore_master kwarg"
+                % (from_branch.tags,),),
+            DeprecationWarning)
+        from_branch.tags.merge_to(to_branch.tags)
 

=== modified file 'bzrlib/tests/blackbox/test_add.py'
--- bzrlib/tests/blackbox/test_add.py	2010-07-16 13:53:26 +0000
+++ bzrlib/tests/blackbox/test_add.py	2010-11-22 03:35:24 +0000
@@ -34,7 +34,7 @@
         standard_tests, tests.condition_isinstance(TestAdd))
     scenarios = [
         ('pre-views', {'branch_tree_format': 'pack-0.92'}),
-        ('view-aware', {'branch_tree_format': 'development6-rich-root'}),
+        ('view-aware', {'branch_tree_format': '2a'}),
         ]
     return tests.multiply_tests(to_adapt, scenarios, result)
 

=== modified file 'bzrlib/tests/blackbox/test_config.py'
--- bzrlib/tests/blackbox/test_config.py	2010-11-05 11:13:05 +0000
+++ bzrlib/tests/blackbox/test_config.py	2010-11-09 16:26:07 +0000
@@ -74,6 +74,40 @@
         super(TestConfigDisplay, self).setUp()
         _t_config.create_configs(self)
 
+    def test_multiline_all_values(self):
+        self.bazaar_config.set_user_option('multiline', '1\n2\n')
+        script.run_script(self, """\
+            $ bzr config -d tree
+            bazaar:
+              multiline = '''1
+            2
+            '''
+            """)
+
+    def test_multiline_value_only(self):
+        self.bazaar_config.set_user_option('multiline', '1\n2\n')
+        script.run_script(self, """\
+            $ bzr config -d tree multiline
+            '''1
+            2
+            '''
+            """)
+
+    def test_list_all_values(self):
+        self.bazaar_config.set_user_option('list', [1, 'a', 'with, a comma'])
+        script.run_script(self, '''\
+            $ bzr config -d tree
+            bazaar:
+              list = 1, a, "with, a comma"
+            ''')
+
+    def test_list_value_only(self):
+        self.bazaar_config.set_user_option('list', [1, 'a', 'with, a comma'])
+        script.run_script(self, '''\
+            $ bzr config -d tree list
+            1, a, "with, a comma"
+            ''')
+
     def test_bazaar_config(self):
         self.bazaar_config.set_user_option('hello', 'world')
         script.run_script(self, '''\
@@ -88,6 +122,7 @@
         script.run_script(self, '''\
             $ bzr config -d tree
             locations:
+              [.../tree]
               hello = world
             branch:
               hello = you
@@ -102,6 +137,33 @@
               hello = world
             ''')
 
+class TestConfigDisplayWithPolicy(tests.TestCaseWithTransport):
+
+    def test_location_with_policy(self):
+        # LocationConfig is the only one dealing with policies so far.
+        self.make_branch_and_tree('tree')
+        config_text = """\
+[%(dir)s]
+url = dir
+url:policy = appendpath
+[%(dir)s/tree]
+url = tree
+""" % {'dir': self.test_dir}
+        # We don't use the config directly so we save it to disk
+        config.LocationConfig.from_string(config_text, 'tree', save=True)
+        # policies are displayed with their options since they are part of
+        # their definition, likewise the path is not appended, we are just
+        # presenting the relevant portions of the config files
+        script.run_script(self, '''\
+            $ bzr config -d tree --all url
+            locations:
+              [.../work/tree]
+              url = tree
+              [.../work]
+              url = dir
+              url:policy = appendpath
+            ''')
+
 
 class TestConfigActive(tests.TestCaseWithTransport):
 
@@ -162,6 +224,7 @@
             $ bzr config -d tree --scope locations hello=world
             $ bzr config -d tree --all hello
             locations:
+              [.../work/tree]
               hello = world
             ''')
 
@@ -197,6 +260,7 @@
             $ bzr config --scope bazaar --remove file
             $ bzr config -d tree --all file
             locations:
+              [.../work/tree]
               file = locations
             branch:
               file = branch
@@ -207,6 +271,7 @@
             $ bzr config -d tree --scope bazaar --remove file
             $ bzr config -d tree --all file
             locations:
+              [.../work/tree]
               file = locations
             branch:
               file = branch
@@ -243,6 +308,7 @@
             $ bzr config -d tree --scope branch --remove file
             $ bzr config -d tree --all file
             locations:
+              [.../work/tree]
               file = locations
             bazaar:
               file = bazaar

=== modified file 'bzrlib/tests/blackbox/test_filtered_view_ops.py'
--- bzrlib/tests/blackbox/test_filtered_view_ops.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_filtered_view_ops.py	2010-11-22 03:35:24 +0000
@@ -28,10 +28,7 @@
 class TestViewFileOperations(tests.TestCaseWithTransport):
 
     def make_abc_tree_with_ab_view(self):
-        # we need to use a specific format because the default format
-        # doesn't support views yet
-        format = bzrdir.format_registry.make_bzrdir('development6-rich-root')
-        wt = self.make_branch_and_tree( '.', format=format)
+        wt = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b', 'c'])
         wt.views.set_view('my', ['a', 'b'])
         return wt
@@ -156,11 +153,8 @@
 class TestViewTreeOperations(tests.TestCaseWithTransport):
 
     def make_abc_tree_and_clone_with_ab_view(self):
-        # we need to use a specific format because the default format
-        # doesn't support views yet
-        format = bzrdir.format_registry.make_bzrdir('development6-rich-root')
         # Build the first tree
-        wt1 = self.make_branch_and_tree('tree_1', format=format)
+        wt1 = self.make_branch_and_tree('tree_1')
         self.build_tree(['tree_1/a', 'tree_1/b', 'tree_1/c'])
         wt1.add(['a', 'b', 'c'])
         wt1.commit("adding a b c")

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2010-05-20 18:23:17 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2010-11-09 04:32:56 +0000
@@ -23,9 +23,9 @@
 
 from bzrlib import (
     branch,
+    branchbuilder,
     bzrdir,
     conflicts,
-    errors,
     merge_directive,
     osutils,
     tests,

=== modified file 'bzrlib/tests/blackbox/test_tags.py'
--- bzrlib/tests/blackbox/test_tags.py	2010-11-10 02:01:33 +0000
+++ bzrlib/tests/blackbox/test_tags.py	2010-11-18 00:22:24 +0000
@@ -132,6 +132,18 @@
         builder.build_commit(message='Commit in fork.', rev_id='fork-1')
         return fork
 
+    def test_merge_without_commit_does_not_propagate_tags_to_master(self):
+        """'bzr merge' alone does not propagate tags to a master branch.
+
+        (If the user runs 'bzr commit', then that is when the tags from the
+        merge are propagated.)
+        """
+        master, child = self.make_master_and_checkout()
+        fork = self.make_fork(master)
+        fork.tags.set_tag('new-tag', fork.last_revision())
+        self.run_bzr(['merge', '../fork'], working_dir='child')
+        self.assertEqual({}, master.tags.get_tag_dict())
+
     def test_commit_in_heavyweight_checkout_copies_tags_to_master(self):
         master, child = self.make_master_and_checkout()
         fork = self.make_fork(master)

=== modified file 'bzrlib/tests/blackbox/test_view.py'
--- bzrlib/tests/blackbox/test_view.py	2009-04-08 03:34:31 +0000
+++ bzrlib/tests/blackbox/test_view.py	2010-11-22 03:35:24 +0000
@@ -23,19 +23,12 @@
 
 class TestViewUI(TestCaseWithTransport):
 
-    def make_branch_and_tree(self):
-        # we need to use a specific format because the default format
-        # doesn't support views yet
-        format = bzrdir.format_registry.make_bzrdir('development6-rich-root')
-        return TestCaseWithTransport.make_branch_and_tree(self, '.',
-            format=format)
-
     def test_view_command_help(self):
         out, err = self.run_bzr('help view')
         self.assertContainsRe(out, 'Manage filtered views')
 
     def test_define_view(self):
-        wt = self.make_branch_and_tree()
+        wt = self.make_branch_and_tree('.')
         # Check definition of a new view
         out, err = self.run_bzr('view a b c')
         self.assertEquals(out, "Using 'my' view: a, b, c\n")
@@ -51,7 +44,7 @@
         self.assertContainsRe(err, "Cannot change the 'off' pseudo view")
 
     def test_list_view(self):
-        wt = self.make_branch_and_tree()
+        wt = self.make_branch_and_tree('.')
         # Check list of the current view
         out, err = self.run_bzr('view')
         self.assertEquals(out, "No current view.\n")
@@ -76,7 +69,7 @@
         self.assertContainsRe(err, "No such view")
 
     def test_delete_view(self):
-        wt = self.make_branch_and_tree()
+        wt = self.make_branch_and_tree('.')
         # Check delete of the current view
         out, err = self.run_bzr('view --delete', retcode=3)
         self.assertContainsRe(err, "No current view to delete")
@@ -101,7 +94,7 @@
         self.assertContainsRe(err, "Both --delete and a file list specified")
 
     def test_switch_view(self):
-        wt = self.make_branch_and_tree()
+        wt = self.make_branch_and_tree('.')
         # Check switching to a named view
         self.run_bzr('view a b c')
         self.run_bzr('view e f --name foo')

=== modified file 'bzrlib/tests/per_branch/test_tags.py'
--- bzrlib/tests/per_branch/test_tags.py	2010-11-11 03:53:12 +0000
+++ bzrlib/tests/per_branch/test_tags.py	2010-11-18 00:43:57 +0000
@@ -96,14 +96,14 @@
         # removed when we merge them
         b2.tags.set_tag('in-destination', 'revid')
         result = b1.tags.merge_to(b2.tags)
-        self.assertEquals(result, [])
+        self.assertEquals(list(result), [])
         self.assertEquals(b2.tags.lookup_tag('in-destination'), 'revid')
         # if there's a conflicting tag, it's reported -- the command line
         # interface will say "these tags couldn't be copied"
         b1.tags.set_tag('conflicts', 'revid-1')
         b2.tags.set_tag('conflicts', 'revid-2')
         result = b1.tags.merge_to(b2.tags)
-        self.assertEquals(result,
+        self.assertEquals(list(result),
             [('conflicts', 'revid-1', 'revid-2')])
         # and it keeps the same value
         self.assertEquals(b2.tags.lookup_tag('conflicts'), 'revid-2')
@@ -227,6 +227,176 @@
         self.assertEqual({'one': 'rev-1-changed'}, b.tags.get_tag_dict())
 
 
+class TestTagsMergeToInCheckouts(per_branch.TestCaseWithBranch):
+    """Tests for checkout.branch.tags.merge_to.
+    
+    In particular this exercises variations in tag conflicts in the master
+    branch and/or the checkout (child).  It may seem strange to have different
+    tags in the child and master, but 'bzr merge' intentionally updates the
+    child and not the master (instead the next 'bzr commit', if the user
+    decides to commit, will update the master).  Also, merge_to in bzr < 2.3
+    didn't propagate changes to the master, and current bzr versions may find
+    themselves operating on checkouts touched by older bzrs
+    
+    So we need to make sure bzr copes gracefully with differing tags in the
+    master versus the child.
+
+    See also <https://bugs.launchpad.net/bzr/+bug/603395>.
+    """
+
+    def setUp(self):
+        super(TestTagsMergeToInCheckouts, self).setUp()
+        branch1 = self.make_branch('tags-probe')
+        if not branch1._format.supports_tags():
+            raise tests.TestSkipped(
+                "format %s doesn't support tags" % branch1._format)
+        branch2 = self.make_branch('bind-probe')
+        try:
+            branch2.bind(branch1)
+        except errors.UpgradeRequired:
+            raise tests.TestNotApplicable(
+                "format %s doesn't support bound branches" % branch2._format)
+
+    def test_merge_to_propagates_tags(self):
+        """merge_to(child) also merges tags to the master."""
+        master = self.make_branch('master')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        other.tags.merge_to(child.tags)
+        self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
+        self.assertEquals('rev-1', master.tags.lookup_tag('foo'))
+
+    def test_ignore_master_disables_tag_propagation(self):
+        """merge_to(child, ignore_master=True) does not merge tags to the
+        master.
+        """
+        master = self.make_branch('master')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        other.tags.merge_to(child.tags, ignore_master=True)
+        self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
+        self.assertRaises(errors.NoSuchTag, master.tags.lookup_tag, 'foo')
+
+    def test_merge_to_overwrite_conflict_in_master(self):
+        """merge_to(child, overwrite=True) overwrites any conflicting tags in
+        the master.
+        """
+        master = self.make_branch('master')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        master.tags.set_tag('foo', 'rev-2')
+        tag_conflicts = other.tags.merge_to(child.tags, overwrite=True)
+        self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
+        self.assertEquals('rev-1', master.tags.lookup_tag('foo'))
+        self.assertLength(0, tag_conflicts)
+
+    def test_merge_to_overwrite_conflict_in_child_and_master(self):
+        """merge_to(child, overwrite=True) overwrites any conflicting tags in
+        both the child and the master.
+        """
+        master = self.make_branch('master')
+        master.tags.set_tag('foo', 'rev-2')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        tag_conflicts = other.tags.merge_to(child.tags, overwrite=True)
+        self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
+        self.assertEquals('rev-1', master.tags.lookup_tag('foo'))
+        self.assertLength(0, tag_conflicts)
+
+    def test_merge_to_conflict_in_child_only(self):
+        """When new_tags.merge_to(child.tags) conflicts with the child but not
+        the master, a conflict is reported and the child receives the new tag.
+        """
+        master = self.make_branch('master')
+        master.tags.set_tag('foo', 'rev-2')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        master.tags.delete_tag('foo')
+        tag_conflicts = other.tags.merge_to(child.tags)
+        # Conflict in child, so it is unchanged.
+        self.assertEquals('rev-2', child.tags.lookup_tag('foo'))
+        # No conflict in the master, so the 'foo' tag equals other's value here.
+        self.assertEquals('rev-1', master.tags.lookup_tag('foo'))
+        # The conflict is reported.
+        self.assertEquals([(u'foo', 'rev-1', 'rev-2')], list(tag_conflicts))
+
+    def test_merge_to_conflict_in_master_only(self):
+        """When new_tags.merge_to(child.tags) conflicts with the master but not
+        the child, a conflict is reported and the child receives the new tag.
+        """
+        master = self.make_branch('master')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        master.tags.set_tag('foo', 'rev-2')
+        tag_conflicts = other.tags.merge_to(child.tags)
+        # No conflict in the child, so the 'foo' tag equals other's value here.
+        self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
+        # Conflict in master, so it is unchanged.
+        self.assertEquals('rev-2', master.tags.lookup_tag('foo'))
+        # The conflict is reported.
+        self.assertEquals([(u'foo', 'rev-1', 'rev-2')], list(tag_conflicts))
+
+    def test_merge_to_same_conflict_in_master_and_child(self):
+        """When new_tags.merge_to(child.tags) conflicts the same way with the
+        master and the child a single conflict is reported.
+        """
+        master = self.make_branch('master')
+        master.tags.set_tag('foo', 'rev-2')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        tag_conflicts = other.tags.merge_to(child.tags)
+        # Both master and child conflict, so both stay as rev-2
+        self.assertEquals('rev-2', child.tags.lookup_tag('foo'))
+        self.assertEquals('rev-2', master.tags.lookup_tag('foo'))
+        # The conflict is reported exactly once, even though it occurs in both
+        # master and child.
+        self.assertEquals([(u'foo', 'rev-1', 'rev-2')], list(tag_conflicts))
+
+    def test_merge_to_different_conflict_in_master_and_child(self):
+        """When new_tags.merge_to(child.tags) conflicts differently in the
+        master and the child both conflicts are reported.
+        """
+        master = self.make_branch('master')
+        master.tags.set_tag('foo', 'rev-2')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        # We use the private method _set_tag_dict because normally bzr tries to
+        # avoid this scenario.
+        child.tags._set_tag_dict({'foo': 'rev-3'})
+        tag_conflicts = other.tags.merge_to(child.tags)
+        # Both master and child conflict, so both stay as they were.
+        self.assertEquals('rev-3', child.tags.lookup_tag('foo'))
+        self.assertEquals('rev-2', master.tags.lookup_tag('foo'))
+        # Both conflicts are reported.
+        self.assertEquals(
+            [(u'foo', 'rev-1', 'rev-2'), (u'foo', 'rev-1', 'rev-3')],
+            sorted(tag_conflicts))
+
+
 class TestUnsupportedTags(per_branch.TestCaseWithBranch):
     """Formats that don't support tags should give reasonable errors."""
 

=== modified file 'bzrlib/tests/per_interrepository/__init__.py'
--- bzrlib/tests/per_interrepository/__init__.py	2010-09-17 04:35:23 +0000
+++ bzrlib/tests/per_interrepository/__init__.py	2010-11-22 03:35:24 +0000
@@ -128,12 +128,6 @@
     add_combo('InterDifferingSerializer',
               groupcompress_repo.RepositoryFormat2a(),
               pack_repo.RepositoryFormatKnitPack6RichRoot())
-    add_combo('InterRepository',
-              groupcompress_repo.RepositoryFormatCHK2(),
-              groupcompress_repo.RepositoryFormat2a())
-    add_combo('InterDifferingSerializer',
-              groupcompress_repo.RepositoryFormatCHK1(),
-              groupcompress_repo.RepositoryFormat2a())
     return result
 
 

=== modified file 'bzrlib/tests/per_intertree/__init__.py'
--- bzrlib/tests/per_intertree/__init__.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/per_intertree/__init__.py	2010-11-22 03:35:24 +0000
@@ -137,8 +137,7 @@
             # -- vila 20090311
             chk_tree_format = WorkingTreeFormat4()
             chk_tree_format._get_matchingbzrdir = \
-                lambda:bzrlib.bzrdir.format_registry.make_bzrdir(
-                    'development6-rich-root')
+                lambda:bzrlib.bzrdir.format_registry.make_bzrdir('2a')
             test_intertree_permutations.append(
                 (InterTree.__name__ + "(CHKInventory)",
                  InterTree,

=== modified file 'bzrlib/tests/per_repository_chk/__init__.py'
--- bzrlib/tests/per_repository_chk/__init__.py	2010-03-31 16:29:25 +0000
+++ bzrlib/tests/per_repository_chk/__init__.py	2010-11-22 03:35:24 +0000
@@ -31,7 +31,7 @@
     RepositoryFormatKnitPack5,
     )
 from bzrlib.repofmt.groupcompress_repo import (
-    RepositoryFormatCHK1,
+    RepositoryFormat2a,
     )
 from bzrlib.tests import (
     multiply_tests,
@@ -61,7 +61,7 @@
         # with the chosen backing format.
         if isinstance(format, remote.RemoteRepositoryFormat):
             with_support = dict(scenario_info)
-            with_support['repository_format'] = RepositoryFormatCHK1()
+            with_support['repository_format'] = RepositoryFormat2a()
             supported_scenarios.append((test_name + "(Supported)", with_support))
             no_support = dict(scenario_info)
             no_support['repository_format'] = RepositoryFormatKnitPack5()

=== modified file 'bzrlib/tests/test_branchbuilder.py'
--- bzrlib/tests/test_branchbuilder.py	2010-02-27 12:27:33 +0000
+++ bzrlib/tests/test_branchbuilder.py	2010-11-18 08:13:01 +0000
@@ -324,6 +324,21 @@
         # should look like it was not modified in the merge
         self.assertEqual('C-id', d_tree.inventory['c-id'].revision)
 
+    def test_set_parent_to_null(self):
+        builder = self.build_a_rev()
+        builder.start_series()
+        self.addCleanup(builder.finish_series)
+        builder.build_snapshot('B-id', [],
+            [('add', ('', None, 'directory', None))])
+        # We should now have a graph:
+        #   A B
+        # And not A => B
+        repo = builder.get_branch().repository
+        self.assertEqual({'A-id': (_mod_revision.NULL_REVISION,),
+                          'B-id': (_mod_revision.NULL_REVISION,),},
+                         repo.get_parent_map(['A-id', 'B-id']))
+
+    
     def test_start_finish_series(self):
         builder = BranchBuilder(self.get_transport().clone('foo'))
         builder.start_series()

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2010-10-29 16:53:22 +0000
+++ bzrlib/tests/test_config.py	2010-11-17 15:52:03 +0000
@@ -21,6 +21,9 @@
 import sys
 import threading
 
+
+from testtools import matchers
+
 #import bzrlib specific imports here
 from bzrlib import (
     branch,
@@ -125,6 +128,49 @@
 """
 
 
+def create_configs(test):
+    """Create configuration files for a given test.
+
+    This requires creating a tree (and populate the ``test.tree`` attribute)
+    and its associated branch and will populate the following attributes:
+
+    - branch_config: A BranchConfig for the associated branch.
+
+    - locations_config : A LocationConfig for the associated branch
+
+    - bazaar_config: A GlobalConfig.
+
+    The tree and branch are created in a 'tree' subdirectory so the tests can
+    still use the test directory to stay outside of the branch.
+    """
+    tree = test.make_branch_and_tree('tree')
+    test.tree = tree
+    test.branch_config = config.BranchConfig(tree.branch)
+    test.locations_config = config.LocationConfig(tree.basedir)
+    test.bazaar_config = config.GlobalConfig()
+
+
+def create_configs_with_file_option(test):
+    """Create configuration files with a ``file`` option set in each.
+
+    This builds on ``create_configs`` and add one ``file`` option in each
+    configuration with a value which allows identifying the configuration file.
+    """
+    create_configs(test)
+    test.bazaar_config.set_user_option('file', 'bazaar')
+    test.locations_config.set_user_option('file', 'locations')
+    test.branch_config.set_user_option('file', 'branch')
+
+
+class TestOptionsMixin:
+
+    def assertOptions(self, expected, conf):
+        # We don't care about the parser (as it will make tests hard to write
+        # and error-prone anyway)
+        self.assertThat([opt[:4] for opt in conf._get_options()],
+                        matchers.Equals(expected))
+
+
 class InstrumentedConfigObj(object):
     """A config obj look-enough-alike to record calls made to it."""
 
@@ -902,7 +948,7 @@
         self.assertIs(None, new_config.get_alias('commit'))
 
 
-class TestLocationConfig(tests.TestCaseInTempDir):
+class TestLocationConfig(tests.TestCaseInTempDir, TestOptionsMixin):
 
     def test_constructs(self):
         my_config = config.LocationConfig('http://example.com')
@@ -1016,6 +1062,21 @@
             'http://www.example.com', 'appendpath_option'),
             config.POLICY_APPENDPATH)
 
+    def test__get_options_with_policy(self):
+        self.get_branch_config('/dir/subdir',
+                               location_config="""\
+[/dir]
+other_url = /other-dir
+other_url:policy = appendpath
+[/dir/subdir]
+other_url = /other-subdir
+""")
+        self.assertOptions(
+            [(u'other_url', u'/other-subdir', u'/dir/subdir', 'locations'),
+             (u'other_url', u'/other-dir', u'/dir', 'locations'),
+             (u'other_url:policy', u'appendpath', u'/dir', 'locations')],
+            self.my_location_config)
+
     def test_location_without_username(self):
         self.get_branch_config('http://www.example.com/ignoreparent')
         self.assertEqual(u'Erik B\u00e5gfors <erik@bagfors.nu>',
@@ -1157,15 +1218,18 @@
         self.assertEqual('bzrlib.tests.test_config.post_commit',
                          self.my_config.post_commit())
 
-    def get_branch_config(self, location, global_config=None):
+    def get_branch_config(self, location, global_config=None,
+                          location_config=None):
         my_branch = FakeBranch(location)
         if global_config is None:
             global_config = sample_config_text
+        if location_config is None:
+            location_config = sample_branches_text
 
         my_global_config = config.GlobalConfig.from_string(global_config,
                                                            save=True)
         my_location_config = config.LocationConfig.from_string(
-            sample_branches_text, my_branch.base, save=True)
+            location_config, my_branch.base, save=True)
         my_config = config.BranchConfig(my_branch)
         self.my_config = my_config
         self.my_location_config = my_config._get_location_config()
@@ -1467,50 +1531,12 @@
         self.assertIs(None, bzrdir_config.get_default_stack_on())
 
 
-def create_configs(test):
-    """Create configuration files for a given test.
-
-    This requires creating a tree (and populate the ``test.tree`` attribute)
-    and its associated branch and will populate the following attributes:
-
-    - branch_config: A BranchConfig for the associated branch.
-
-    - locations_config : A LocationConfig for the associated branch
-
-    - bazaar_config: A GlobalConfig.
-
-    The tree and branch are created in a 'tree' subdirectory so the tests can
-    still use the test directory to stay outside of the branch.
-    """
-    tree = test.make_branch_and_tree('tree')
-    test.tree = tree
-    test.branch_config = config.BranchConfig(tree.branch)
-    test.locations_config = config.LocationConfig(tree.basedir)
-    test.bazaar_config = config.GlobalConfig()
-
-
-def create_configs_with_file_option(test):
-    """Create configuration files with a ``file`` option set in each.
-
-    This builds on ``create_configs`` and add one ``file`` option in each
-    configuration with a value which allows identifying the configuration file.
-    """
-    create_configs(test)
-    test.bazaar_config.set_user_option('file', 'bazaar')
-    test.locations_config.set_user_option('file', 'locations')
-    test.branch_config.set_user_option('file', 'branch')
-
-
-class TestConfigGetOptions(tests.TestCaseWithTransport):
+class TestConfigGetOptions(tests.TestCaseWithTransport, TestOptionsMixin):
 
     def setUp(self):
         super(TestConfigGetOptions, self).setUp()
         create_configs(self)
 
-    def assertOptions(self, expected, conf):
-        actual = list(conf._get_options())
-        self.assertEqual(expected, actual)
-
     # One variable in none of the above
     def test_no_variable(self):
         # Using branch should query branch, locations and bazaar
@@ -1559,16 +1585,12 @@
             self.branch_config)
 
 
-class TestConfigRemoveOption(tests.TestCaseWithTransport):
+class TestConfigRemoveOption(tests.TestCaseWithTransport, TestOptionsMixin):
 
     def setUp(self):
         super(TestConfigRemoveOption, self).setUp()
         create_configs_with_file_option(self)
 
-    def assertOptions(self, expected, conf):
-        actual = list(conf._get_options())
-        self.assertEqual(expected, actual)
-
     def test_remove_in_locations(self):
         self.locations_config.remove_user_option('file', self.tree.basedir)
         self.assertOptions(

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2010-07-29 04:07:27 +0000
+++ bzrlib/tests/test_merge.py	2010-11-18 08:18:19 +0000
@@ -1270,6 +1270,26 @@
         self.assertEqual(['B-id', 'C-id', 'F-id'],
                          [t.get_revision_id() for t in merger._lca_trees])
 
+    def test_find_base_new_root_criss_cross(self):
+        # A   B
+        # |\ /|
+        # | X |
+        # |/ \|
+        # C   D
+        
+        builder = self.get_builder()
+        builder.build_snapshot('A-id', None,
+            [('add', ('', None, 'directory', None))])
+        builder.build_snapshot('B-id', [],
+            [('add', ('', None, 'directory', None))])
+        builder.build_snapshot('D-id', ['A-id', 'B-id'], [])
+        builder.build_snapshot('C-id', ['A-id', 'B-id'], [])
+        merger = self.make_Merger(builder, 'D-id')
+        self.assertEqual('A-id', merger.base_rev_id)
+        self.assertTrue(merger._is_criss_cross)
+        self.assertEqual(['A-id', 'B-id'], [t.get_revision_id()
+                                            for t in merger._lca_trees])
+
     def test_no_criss_cross_passed_to_merge_type(self):
         class LCATreesMerger(LoggingMerger):
             supports_lca_trees = True

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2010-08-29 14:32:45 +0000
+++ bzrlib/tests/test_msgeditor.py	2010-11-11 13:45:02 +0000
@@ -301,9 +301,12 @@
     def test__create_temp_file_with_commit_template_in_unicode_dir(self):
         self.requireFeature(tests.UnicodeFilenameFeature)
         if hasattr(self, 'info'):
-            os.mkdir(self.info['directory'])
-            os.chdir(self.info['directory'])
-            msgeditor._create_temp_file_with_commit_template('infotext')
+            tmpdir = self.info['directory']
+            os.mkdir(tmpdir)
+            # Force the creation of temp file in a directory whose name
+            # requires some encoding support
+            msgeditor._create_temp_file_with_commit_template('infotext',
+                                                             tmpdir=tmpdir)
         else:
             raise TestNotApplicable('Test run elsewhere with non-ascii data.')
 

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2010-08-02 18:36:31 +0000
+++ bzrlib/tests/test_remote.py	2010-11-22 03:35:24 +0000
@@ -1867,7 +1867,7 @@
 class TestRepositoryFormat(TestRemoteRepository):
 
     def test_fast_delta(self):
-        true_name = groupcompress_repo.RepositoryFormatCHK1().network_name()
+        true_name = groupcompress_repo.RepositoryFormat2a().network_name()
         true_format = RemoteRepositoryFormat()
         true_format._network_name = true_name
         self.assertEqual(True, true_format.fast_deltas)

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2010-08-05 19:25:52 +0000
+++ bzrlib/tests/test_repository.py	2010-11-22 22:27:58 +0000
@@ -586,7 +586,7 @@
                                 ]
         repo_a = self.make_repository('a')
         repo_b = self.make_repository('b')
-        is_compatible = repository.InterWeaveRepo.is_compatible
+        is_compatible = weaverepo.InterWeaveRepo.is_compatible
         for source in incompatible_formats:
             # force incompatible left then right
             repo_a._format = source
@@ -598,7 +598,7 @@
             for target in formats:
                 repo_b._format = target
                 self.assertTrue(is_compatible(repo_a, repo_b))
-        self.assertEqual(repository.InterWeaveRepo,
+        self.assertEqual(weaverepo.InterWeaveRepo,
                          repository.InterRepository.get(repo_a,
                                                         repo_b).__class__)
 
@@ -972,8 +972,7 @@
 
     def setUp(self):
         super(TestDevelopment6FindParentIdsOfRevisions, self).setUp()
-        self.builder = self.make_branch_builder('source',
-            format='development6-rich-root')
+        self.builder = self.make_branch_builder('source')
         self.builder.start_series()
         self.builder.build_snapshot('initial', None,
             [('add', ('', 'tree-root', 'directory', None))])

=== modified file 'bzrlib/tests/test_tag.py'
--- bzrlib/tests/test_tag.py	2010-03-25 09:39:03 +0000
+++ bzrlib/tests/test_tag.py	2010-11-18 00:43:57 +0000
@@ -111,15 +111,18 @@
         # conflicting merge
         a.tags.set_tag('tag-2', 'z')
         conflicts = a.tags.merge_to(b.tags)
-        self.assertEqual(conflicts, [('tag-2', 'z', 'y')])
+        self.assertEqual(list(conflicts), [('tag-2', 'z', 'y')])
         self.assertEqual('y', b.tags.lookup_tag('tag-2'))
         # overwrite conflicts
         conflicts = a.tags.merge_to(b.tags, overwrite=True)
-        self.assertEqual(conflicts, [])
+        self.assertEqual(list(conflicts), [])
         self.assertEqual('z', b.tags.lookup_tag('tag-2'))
 
 
 class TestTagsInCheckouts(TestCaseWithTransport):
+    """Tests for how tags are synchronised between the master and child branch
+    of a checkout.
+    """
 
     def test_update_tag_into_checkout(self):
         # checkouts are directly connected to the tags of their master branch:

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2010-10-15 14:21:03 +0000
+++ bzrlib/tests/test_transform.py	2010-11-19 08:34:39 +0000
@@ -3272,15 +3272,20 @@
                                                policy)
 
     def _prepare_orphan(self, wt):
-        self.build_tree(['dir/', 'dir/foo'])
-        wt.add(['dir'], ['dir-id'])
-        wt.commit('add dir')
+        self.build_tree(['dir/', 'dir/file', 'dir/foo'])
+        wt.add(['dir', 'dir/file'], ['dir-id', 'file-id'])
+        wt.commit('add dir and file ignoring foo')
         tt = transform.TreeTransform(wt)
         self.addCleanup(tt.finalize)
+        # dir and bar are deleted
         dir_tid = tt.trans_id_tree_path('dir')
+        file_tid = tt.trans_id_tree_path('dir/file')
         orphan_tid = tt.trans_id_tree_path('dir/foo')
+        tt.delete_contents(file_tid)
+        tt.unversion_file(file_tid)
         tt.delete_contents(dir_tid)
         tt.unversion_file(dir_tid)
+        # There should be a conflict because dir still contain foo
         raw_conflicts = tt.find_conflicts()
         self.assertLength(1, raw_conflicts)
         self.assertEqual(('missing parent', 'new-1'), raw_conflicts[0])
@@ -3290,7 +3295,13 @@
         wt = self.make_branch_and_tree('.')
         self._set_orphan_policy(wt, 'move')
         tt, orphan_tid = self._prepare_orphan(wt)
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
         remaining_conflicts = resolve_conflicts(tt)
+        self.assertEquals(['dir/foo has been orphaned in bzr-orphans'],
+                          warnings)
         # Yeah for resolved conflicts !
         self.assertLength(0, remaining_conflicts)
         # We have a new orphan

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2010-11-10 16:24:31 +0000
+++ bzrlib/transform.py	2010-11-24 16:01:57 +0000
@@ -703,27 +703,29 @@
         return conflicts
 
     def _parent_type_conflicts(self, by_parent):
-        """parents must have directory 'contents'."""
+        """Children must have a directory parent"""
         conflicts = []
         for parent_id, children in by_parent.iteritems():
             if parent_id is ROOT_PARENT:
                 continue
-            if not self._any_contents(children):
+            no_children = True
+            for child_id in children:
+                if self.final_kind(child_id) is not None:
+                    no_children = False
+                    break
+            if no_children:
                 continue
+            # There is at least a child, so we need an existing directory to
+            # contain it.
             kind = self.final_kind(parent_id)
             if kind is None:
+                # The directory will be deleted
                 conflicts.append(('missing parent', parent_id))
             elif kind != "directory":
+                # Meh, we need a *directory* to put something in it
                 conflicts.append(('non-directory parent', parent_id))
         return conflicts
 
-    def _any_contents(self, trans_ids):
-        """Return true if any of the trans_ids, will have contents."""
-        for trans_id in trans_ids:
-            if self.final_kind(trans_id) is not None:
-                return True
-        return False
-
     def _set_executability(self, path, trans_id):
         """Set the executability of versioned files """
         if supports_executable():
@@ -820,9 +822,14 @@
         """
         orphans = []
         # Find the potential orphans, stop if one item should be kept
-        for c in self.by_parent()[dir_id]:
-            if self.final_file_id(c) is None:
-                orphans.append(c)
+        for child_tid in self.by_parent()[dir_id]:
+            if child_tid in self._removed_contents:
+                # The child is removed as part of the transform. Since it was
+                # versioned before, it's not an orphan
+                continue
+            elif self.final_file_id(child_tid) is None:
+                # The child is not versioned
+                orphans.append(child_tid)
             else:
                 # We have a versioned file here, searching for orphans is
                 # meaningless.

=== modified file 'bzrlib/transport/ssh.py'
--- bzrlib/transport/ssh.py	2010-10-07 12:45:51 +0000
+++ bzrlib/transport/ssh.py	2010-11-18 00:22:24 +0000
@@ -363,13 +363,14 @@
             # This platform doesn't support socketpair(), so just use ordinary
             # pipes instead.
             stdin = stdout = subprocess.PIPE
-            sock = None
+            my_sock, subproc_sock = None, None
         else:
             stdin = stdout = subproc_sock
-            sock = my_sock
         proc = subprocess.Popen(argv, stdin=stdin, stdout=stdout,
                                 **os_specific_subprocess_params())
-        return SSHSubprocessConnection(proc, sock=sock)
+        if subproc_sock is not None:
+            subproc_sock.close()
+        return SSHSubprocessConnection(proc, sock=my_sock)
 
     def connect_sftp(self, username, password, host, port):
         try:
@@ -653,25 +654,24 @@
 import weakref
 _subproc_weakrefs = set()
 
-def _close_ssh_proc(proc):
+def _close_ssh_proc(proc, sock):
     """Carefully close stdin/stdout and reap the SSH process.
 
     If the pipes are already closed and/or the process has already been
     wait()ed on, that's ok, and no error is raised.  The goal is to do our best
     to clean up (whether or not a clean up was already tried).
     """
-    dotted_names = ['stdin.close', 'stdout.close', 'wait']
-    for dotted_name in dotted_names:
-        attrs = dotted_name.split('.')
-        try:
-            obj = proc
-            for attr in attrs:
-                obj = getattr(obj, attr)
-        except AttributeError:
-            # It's ok for proc.stdin or proc.stdout to be None.
-            continue
-        try:
-            obj()
+    funcs = []
+    for closeable in (proc.stdin, proc.stdout, sock):
+        # We expect that either proc (a subprocess.Popen) will have stdin and
+        # stdout streams to close, or that we will have been passed a socket to
+        # close, with the option not in use being None.
+        if closeable is not None:
+            funcs.append(closeable.close)
+    funcs.append(proc.wait)
+    for func in funcs:
+        try:
+            func()
         except OSError:
             # It's ok for the pipe to already be closed, or the process to
             # already be finished.
@@ -716,7 +716,7 @@
         # to avoid leaving processes lingering indefinitely.
         def terminate(ref):
             _subproc_weakrefs.remove(ref)
-            _close_ssh_proc(proc)
+            _close_ssh_proc(proc, sock)
         _subproc_weakrefs.add(weakref.ref(self, terminate))
 
     def send(self, data):
@@ -732,7 +732,7 @@
             return os.read(self.proc.stdout.fileno(), count)
 
     def close(self):
-        _close_ssh_proc(self.proc)
+        _close_ssh_proc(self.proc, self._sock)
 
     def get_sock_or_pipes(self):
         if self._sock is not None:

=== modified file 'doc/en/release-notes/bzr-2.3.txt'
--- doc/en/release-notes/bzr-2.3.txt	2010-11-12 09:35:46 +0000
+++ doc/en/release-notes/bzr-2.3.txt	2010-11-24 16:01:57 +0000
@@ -53,10 +53,23 @@
 * Better message if there is an error while setting ownership of
   ``.bazaar`` directory. (Parth Malwankar, #657553)
 
+* ``bzr config`` properly displays list values. (Vincent Ladeuil, #672382)
+
+* ``bzr config`` will now respect option policies when displaying the value
+  and display the definition sections when appropriate.
+  (Vincent Ladeuil, #671050)
+
+* Don't create commit message files in the current directory to avoid nasty
+  interactions with emacs (which tries to establish the status of the file
+  during the commit which breaks on windows). (Vincent Ladeuil, #673637)
+
 * ``bzr resolve --take-other <file>`` will not crash anymore if ``<file>``
   is involved in a text conflict (but the conflict is still not
   resolved). (Vincent Ladeuil, #646961)
 
+* Merge will now correctly locate a lca where there is a criss-cross merge
+  of a new root. (Gary van der Merwe, #588698)
+
 * Report error if non-ASCII command option given. (Rory Yorke, #140563)
 
 Documentation
@@ -76,6 +89,14 @@
 .. Major internal changes, unlikely to be visible to users or plugin 
    developers, but interesting for bzr developers.
 
+* ``BranchBuilder.build_snapshot`` now accepts parent_ids == [].
+  This can be used to create a new root in the graph. (Gary van der Merwe)
+
+* Old repository development formats
+  RepositoryFormatCHK1 and RepositoryFormatCHK2 have been removed, and so
+  have the corresponding metadir format options ``development-rich-root``,
+  ``development6-rich-root``, and ``development7-rich-root``.
+
 Testing
 *******
 
@@ -162,6 +183,11 @@
   ``bzrdir.open_repository()`` to propagate.  This is unhelpful at best,
   and at worst can trigger infinite loops in callers.  (Andrew Bennetts)
   
+* The ``branch.tags.merge_to(target_branch)`` API used by plugins such as
+  ``bzr-builddeb`` now propagates changes to the master branch of the
+  target branch (if there is one).  This makes it consistent with the
+  other tag APIs.  (Andrew Bennetts, #603395)
+
 * Windows installers no longer requires the Microsoft vcredist to be
   installed. (Martin [gz], Gary van der Merwe, #632465)
 

=== modified file 'doc/en/whats-new/whats-new-in-2.3.txt'
--- doc/en/whats-new/whats-new-in-2.3.txt	2010-11-10 16:24:31 +0000
+++ doc/en/whats-new/whats-new-in-2.3.txt	2010-11-24 16:01:57 +0000
@@ -20,6 +20,13 @@
 Changed Behaviour
 *****************
 
+* Support for some old development formats have been removed:
+  ``development-rich-root``, ``development6-rich-root``, and
+  ``development7-rich-root``.  These formats were always labelled experimental
+  and not used unless the user specifically asked for them.  If you have
+  repositories using these old formats you should upgrade them to ``2a`` using
+  Bazaar 2.2.  (Andrew Bennetts)
+
 * The default ``ignore`` file created by Bazaar will contain ``__pycache__``,
   which is the name of the directory that will be used by Python to store
   bytecode files.

